require "fragments/version"
require "fragments/config"
require "fragments/requester"
require "fragments/helpers/controller"

# Top level module for the Fragments library
#
# This module should be considered as a Facade, so some internal functionality
# should not be used from the outside. So, use an API, which is provided by this
# module instead!
module Fragments
  # Get or set a configuration for the library
  def self.config(&block)
    @config ||= ::Fragments::Config.new
    block_given? ? yield(@config) : @config
  end

  # Specifies either should fragments be used or not
  #
  # @return [Boolean] - a value which specifies enableness
  def self.enabled?
    config.is_enabled
  end

  # Provides a html structure for a fragment with a specified code
  #
  # @param code [String] - a code of a fragment which you want to request
  # @param request [Request] - a reference to an object which represents an income HTTP request
  # @return [String] a html structure of a needed fragment
  def self.get_html(code, request)
    ::Fragments::Requester.call(
      fragment_code: code,
      current_host: ::Fragments.config.host,
      cookies: request.headers["HTTP_COOKIE"],
      current_url: request.original_url
    )
  rescue Exception => e
    ""
  end
end
