require 'open-uri'
require 'net/http'
require 'net/https'

module Fragments
  class Requester
    attr_reader :fragment_code, :current_host, :cookies, :current_url

    def initialize(fragment_code:, current_host:, cookies:, current_url:)
      @fragment_code = fragment_code
      @current_host = current_host
      @cookies = cookies
      @current_url = current_url
    end

    def self.call(*opts, &block)
      new(*opts).call(&block)
    end

    def call
      uri = build_uri
      http = Net::HTTP.new(uri.host, uri.port)
      get_request = Net::HTTP::Get.new(uri.path)
      apply_https(http, uri)
      apply_headers(get_request)
      get_response_body(http, get_request)
    end

    private

    def build_uri
      path = current_host + '/' + fragment_code;
      URI.parse(path)
    end

    def apply_https(http, uri)
      return if uri.scheme != "https"
      http.use_ssl = true
      http.verify_mode = OpenSSL::SSL::VERIFY_NONE
    end

    def apply_headers(request)
      request.add_field("Cookie", cookies)
      request.add_field("frontend-uri", fetch_fragment_uri_prefix)
      request.add_field("current-url", current_url)
      request.add_field("host", current_host.gsub(/^https?\:\/\/(www.)?/,''))
    end

    def get_response_body(http, request)
      get_response = http.request(request)
      get_response.body
    end

    def fetch_fragment_uri_prefix
      ::Fragments.config.uri_prefix(I18n.locale)
    end
  end
end
